##############################################################################
#
# Copyright (c) 2001-2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Reusable functionality for testing site-related code
"""
import re

import zope.component
import zope.component.hooks
import zope.component.interfaces
import zope.container.interfaces
import zope.container.testing
from zope.interface import Interface
from zope.interface.interfaces import IComponentLookup
from zope.testing import renormalizing

import zope.site.site
from zope.site import LocalSiteManager
from zope.site import SiteManagerAdapter
from zope.site.folder import rootFolder


checker = renormalizing.RENormalizing([
    # Python 3 unicode removed the "u".
    (re.compile("u('.*?')"),
     r"\1"),
    (re.compile('u(".*?")'),
     r"\1"),
])


def createSiteManager(folder, setsite=False):
    if not zope.component.interfaces.ISite.providedBy(folder):
        folder.setSiteManager(LocalSiteManager(folder))
    if setsite:  # pragma: no cover
        zope.component.hooks.setSite(folder)
    return folder.getSiteManager()


def addUtility(sitemanager, name, iface, utility, suffix=''):
    """Add a utility to a site manager

    This helper function is useful for tests that need to set up utilities.
    """
    folder_name = (name or (iface.__name__ + 'Utility')) + suffix
    default = sitemanager['default']
    default[folder_name] = utility
    utility = default[folder_name]
    sitemanager.registerUtility(utility, iface, name)
    return utility


def siteSetUp(site=False):
    zope.container.testing.setUp()
    zope.component.hooks.setHooks()

    zope.component.provideAdapter(
        SiteManagerAdapter, (Interface,), IComponentLookup)

    if site:
        site = rootFolder()
        createSiteManager(site, setsite=True)
        return site


def siteTearDown():
    zope.container.testing.tearDown()
    zope.component.hooks.resetHooks()
    zope.component.hooks.setSite()





import doctest
import unittest

from zope.site.folder import Folder
from zope.site.testing import checker
from zope.site.testing import siteSetUp
from zope.site.testing import siteTearDown
from zope.site.tests.test_site import TestSiteManagerContainer


def setUp(test=None):
    siteSetUp()


def tearDown(test=None):
    siteTearDown()


class FolderTest(TestSiteManagerContainer):

    def makeTestObject(self):
        return Folder()


class TestRootFolder(unittest.TestCase):

    def test_IRoot_before_IContainer_rootFolder(self):
        from zope.container.interfaces import IContainer
        from zope.interface import providedBy
        from zope.location.interfaces import IRoot

        from zope.site.folder import rootFolder

        folder = rootFolder()
        provides = list(providedBy(folder).flattened())

        iroot = provides.index(IRoot)
        container = provides.index(IContainer)

        self.assertLess(iroot, container)

    def test_IRoot_before_IContainer_IRootFolder(self):
        from zope.container.interfaces import IContainer
        from zope.location.interfaces import IRoot

        from zope.site.interfaces import IRootFolder

        provides = list(IRootFolder.__iro__)

        iroot = provides.index(IRoot)
        container = provides.index(IContainer)

        self.assertLess(iroot, container)


def test_suite():
    flags = doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
    return unittest.TestSuite((
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        doctest.DocTestSuite('zope.site.folder',
                             setUp=setUp, tearDown=tearDown),
        doctest.DocFileSuite("folder.txt",
                             setUp=setUp, tearDown=tearDown,
                             checker=checker, optionflags=flags),
    ))



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Local Component Architecture
"""

from zope.site.site import LocalSiteManager
from zope.site.site import SiteManagementFolder
from zope.site.site import SiteManagerAdapter
from zope.site.site import SiteManagerContainer
from zope.site.site import changeSiteConfigurationAfterMove
from zope.site.site import clearThreadSiteSubscriber
from zope.site.site import threadSiteSubscriber


__all__ = [
    'SiteManagerContainer',
    'SiteManagementFolder',
    'SiteManagerAdapter',
    'LocalSiteManager',
    'threadSiteSubscriber',
    'clearThreadSiteSubscriber',
    'changeSiteConfigurationAfterMove',
]


# BBB. Remove in Version 5.0
from zope.component import getNextUtility
from zope.component import queryNextUtility
from zope.deprecation import deprecated


getNextUtility = deprecated(
    getNextUtility,
    '``zope.site.getNextUtility`` is deprecated '
    'and will be removed in zope.site 5.0. '
    'Use it from ``zope.component`` instead.')
queryNextUtility = deprecated(
    queryNextUtility,
    '``zope.site.queryNextUtility`` is deprecated '
    'and will be removed in zope.site 5.0. '
    'Use it from ``zope.component`` instead.')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Site and Local Site Manager implementation

A local site manager has a number of roles:

  - A local site manager, that provides a local adapter and utility registry.

  - A place to do TTW development and/or to manage database-based code.

  - A registry for persistent modules.  The Zope 3 import hook uses the
    SiteManager to search for modules.
"""

import zope.component
import zope.component.hooks
import zope.component.interfaces
import zope.event
import zope.interface
import zope.lifecycleevent.interfaces
import zope.location
import zope.location.interfaces
from zope.component.hooks import setSite
from zope.component.interfaces import ISite
from zope.component.persistentregistry import PersistentAdapterRegistry
from zope.component.persistentregistry import PersistentComponents
from zope.container.btree import BTreeContainer
from zope.container.contained import Contained
from zope.deprecation import deprecated
from zope.filerepresentation.interfaces import IDirectoryFactory
from zope.interface.interfaces import ComponentLookupError
from zope.interface.interfaces import IComponentLookup
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location.interfaces import ILocationInfo
from zope.location.interfaces import IRoot

from zope.site import interfaces


# BBB. Remove in Version 5.0 including imports
setSite = deprecated(
    setSite,
    '``zope.site.site.setSite`` is deprecated '
    'and will be removed in zope.site Version 5.0. '
    'Use it from ``zope.component.hooks`` instead.')  # noqa


@zope.interface.implementer(interfaces.ISiteManagementFolder)
class SiteManagementFolder(BTreeContainer):
    """Implementation of a :class:`~.ISiteManagementFolder`"""


@zope.interface.implementer(IDirectoryFactory)
class SMFolderFactory(object):
    """
    Implementation of a :class:`~.IDirectoryFactory` that creates
    :class:`SiteManagementFolder`
    """

    def __init__(self, context):
        self.context = context

    def __call__(self, name):
        return SiteManagementFolder()


@zope.interface.implementer(zope.component.interfaces.IPossibleSite)
class SiteManagerContainer(Contained):
    """Implement access to the site manager (++etc++site).

    This is a mix-in that implements the :class:`~.IPossibleSite`
    interface; for example, it is used by the Folder implementation.
    """

    _sm = None

    def getSiteManager(self):
        if self._sm is not None:
            return self._sm
        raise ComponentLookupError('no site manager defined')

    def setSiteManager(self, sm):
        # pylint:disable=no-value-for-parameter
        if ISite.providedBy(self):
            raise TypeError("Already a site")

        if IComponentLookup.providedBy(sm):
            self._sm = sm
        else:
            raise ValueError('setSiteManager requires an IComponentLookup')

        zope.interface.directlyProvides(
            self, zope.component.interfaces.ISite,
            zope.interface.directlyProvidedBy(self))
        zope.event.notify(interfaces.NewLocalSite(sm))


def _findNextSiteManager(site):
    while True:
        if IRoot.providedBy(site):   # pylint:disable=no-value-for-parameter
            # we're the root site, return None
            return None

        try:
            # pylint:disable=no-value-for-parameter, too-many-function-args
            # pylint:disable=assignment-from-no-return
            site = ILocationInfo(site).getParent()
        except TypeError:
            # there was not enough context; probably run from a test
            return None

        if ISite.providedBy(site):  # pylint:disable=no-value-for-parameter
            return site.getSiteManager()


class _LocalAdapterRegistry(
        PersistentAdapterRegistry,
        zope.location.Location,
):
    pass


@zope.interface.implementer(interfaces.ILocalSiteManager)
class LocalSiteManager(BTreeContainer,
                       PersistentComponents):
    """Local Site Manager (:class:`~.ILocalSiteManager`) implementation"""

    subs = ()

    def _setBases(self, bases):

        # Update base subs
        for base in self.__bases__:
            if ((base not in bases)  # pragma: no cover
                    # pylint:disable=no-value-for-parameter
                    and interfaces.ILocalSiteManager.providedBy(base)):
                base.removeSub(self)

        for base in bases:
            if ((base not in self.__bases__)
                    # pylint:disable=no-value-for-parameter
                    and interfaces.ILocalSiteManager.providedBy(base)):
                base.addSub(self)

        super(LocalSiteManager, self)._setBases(bases)

    def __init__(self, site, default_folder=True):
        BTreeContainer.__init__(self)
        PersistentComponents.__init__(self)

        # Locate the site manager
        self.__parent__ = site
        self.__name__ = '++etc++site'

        # Set base site manager
        next_sm = _findNextSiteManager(site)
        if next_sm is None:
            next_sm = zope.component.getGlobalSiteManager()
        self.__bases__ = (next_sm, )

        # Setup default site management folder if requested
        if default_folder:
            folder = SiteManagementFolder()
            zope.event.notify(ObjectCreatedEvent(folder))
            self['default'] = folder

    def _init_registries(self):
        self.adapters = _LocalAdapterRegistry()
        self.utilities = _LocalAdapterRegistry()
        self.adapters.__parent__ = self.utilities.__parent__ = self
        self.adapters.__name__ = u'adapters'
        self.utilities.__name__ = u'utilities'

    def _p_repr(self):
        return PersistentComponents.__repr__(self)

    def addSub(self, sub):
        """See :meth:`zope.site.interfaces.ILocalSiteManager.addSub`"""
        self.subs += (sub, )

    def removeSub(self, sub):
        """See :meth:`zope.site.interfaces.ILocalSiteManager.removeSub`"""
        self.subs = tuple(
            [s for s in self.subs if s is not sub])


def threadSiteSubscriber(ob, event):
    """A multi-subscriber to `zope.component.interfaces.ISite` and
    `zope.traversing.interfaces.BeforeTraverseEvent`.

    Sets the 'site' thread global if the object traversed is a site.

    .. note::

       The ``configure.zcml`` included in this package does
       *not* install this subscriber. That must be configured separately.
       ``zope.app.publication`` includes such configuration.
    """
    zope.component.hooks.setSite(ob)


def clearThreadSiteSubscriber(event):
    """A subscriber to `zope.publisher.interfaces.EndRequestEvent`

    Cleans up the site thread global after the request is processed.

    .. note::

        The ``configure.zcml`` included in this package does *not*
        install this subscriber. That must be configured separately.
        ``zope.app.publication`` includes such configuration.
    """
    clearSite()


# Clear the site thread global
clearSite = zope.component.hooks.setSite
try:
    from zope.testing.cleanup import addCleanUp
except ImportError:  # pragma: no cover
    pass
else:
    addCleanUp(clearSite)


@zope.component.adapter(zope.interface.Interface)
@zope.interface.implementer(IComponentLookup)
def SiteManagerAdapter(ob):
    """An adapter from :class:`~.ILocation` to :class:`~.IComponentLookup`.

    The ILocation is interpreted flexibly, we just check for
    ``__parent__``.
    """
    current = ob
    while True:
        if ISite.providedBy(current):  # pylint:disable=no-value-for-parameter
            return current.getSiteManager()
        current = getattr(current, '__parent__', None)
        if current is None:
            # It is not a location or has no parent, so we return the global
            # site manager
            return zope.component.getGlobalSiteManager()


def changeSiteConfigurationAfterMove(site, event):
    """
    After a site is (re-)moved, its site manager links have to be
    updated.

    Subscriber to :class:`~.ISite` objects in a :class:`~.IObjectMovedEvent`.
    """
    local_sm = site.getSiteManager()
    if event.newParent is not None:
        next_sm = _findNextSiteManager(site)
        if next_sm is None:
            next_sm = zope.component.getGlobalSiteManager()
        local_sm.__bases__ = (next_sm, )
    else:
        local_sm.__bases__ = ()


@zope.component.adapter(
    SiteManagerContainer,
    zope.lifecycleevent.interfaces.IObjectMovedEvent)
def siteManagerContainerRemoved(container, event):
    # The relation between SiteManagerContainer and LocalSiteManager is a
    # kind of containment hierarchy, but it is not expressed via containment,
    # but rather via an attribute (_sm).
    #
    # When the parent is deleted, this needs to be propagated to the children,
    # and since we don't have "real" containment, we need to do that manually.

    try:
        sm = container.getSiteManager()
    except ComponentLookupError:
        pass
    else:
        zope.component.handle(sm, event)



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Registration Tests
"""
__docformat__ = "reStructuredText"

import unittest

import persistent
import transaction
import zope.component.globalregistry
import zope.component.testing as placelesssetup
import zope.container.contained

import zope.site
from zope import interface


# test class for testing data conversion
class IFoo(interface.Interface):
    pass


@interface.implementer(IFoo)
class Foo(persistent.Persistent, zope.container.contained.Contained):
    name = ''

    def __init__(self, name=''):
        self.name = name


class GlobalRegistry:
    pass


base = zope.component.globalregistry.GlobalAdapterRegistry(
    GlobalRegistry, 'adapters')
GlobalRegistry.adapters = base


def clear_base():
    base.__init__(GlobalRegistry, 'adapters')


class Test(unittest.TestCase):

    def setUp(self):
        placelesssetup.setUp(self)

    def tearDown(self):
        placelesssetup.tearDown(self)

    def test_deghostification_of_persistent_adapter_registries(self):

        # Note that this test duplicates one from zope.component.tests.
        # We should be able to get rid of this one when we get rid of
        # __setstate__ implementation we have in back35.

        # We want to make sure that we see updates correctly.

        from ZODB.MappingStorage import DB
        db = DB()
        tm1 = transaction.TransactionManager()
        c1 = db.open(transaction_manager=tm1)
        r1 = zope.site.site._LocalAdapterRegistry((base,))
        r2 = zope.site.site._LocalAdapterRegistry((r1,))
        c1.root()[1] = r1
        c1.root()[2] = r2
        tm1.commit()
        r1._p_deactivate()
        r2._p_deactivate()

        tm2 = transaction.TransactionManager()
        c2 = db.open(transaction_manager=tm2)
        r1 = c2.root()[1]
        r2 = c2.root()[2]

        self.assertIsNone(r1.lookup((), IFoo, ''))

        foo_blank = Foo('')
        base.register((), IFoo, '', foo_blank)
        self.assertEqual(r1.lookup((), IFoo, ''),
                         foo_blank)

        self.assertIsNone(r2.lookup((), IFoo, '1'))

        foo_1 = Foo('1')
        r1.register((), IFoo, '1', foo_1)

        self.assertEqual(r2.lookup((), IFoo, '1'),
                         foo_1)

        self.assertIsNone(r1.lookup((), IFoo, '2'))
        self.assertIsNone(r2.lookup((), IFoo, '2'))

        foo_2 = Foo('2')
        base.register((), IFoo, '2', foo_2)

        self.assertEqual(r1.lookup((), IFoo, '2'),
                         foo_2)

        self.assertEqual(r2.lookup((), IFoo, '2'),
                         foo_2)

        # Cleanup:

        db.close()
        clear_base()



#############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import zope.component.interfaces
import zope.container.folder
from zope.interface import directlyProvides
from zope.interface import implementer

from zope.site.interfaces import IFolder
from zope.site.interfaces import IRootFolder
from zope.site.site import SiteManagerContainer


@implementer(IFolder)
class Folder(zope.container.folder.Folder, SiteManagerContainer):
    """Implementation of :class:`zope.site.interfaces.IFolder`"""


def rootFolder():
    """Factory for a :class:`zope.site.interfaces.IRootFolder`"""
    f = Folder()
    directlyProvides(f, IRootFolder)
    return f


class FolderSublocations(object):
    """
    Adapter for an :class:`zope.site.interfaces.IFolder` to
    :class:`zope.location.interfaces.ISublocations`.

    The subobjects of a folder include it's contents and it's site
    manager if it is a site::

        >>> from zope.container.contained import Contained
        >>> folder = Folder()
        >>> folder['ob1'] = Contained()
        >>> folder['ob2'] = Contained()
        >>> folder['ob3'] = Contained()
        >>> subs = list(FolderSublocations(folder).sublocations())
        >>> subs.remove(folder['ob1'])
        >>> subs.remove(folder['ob2'])
        >>> subs.remove(folder['ob3'])
        >>> subs
        []

        >>> sm = Contained()
        >>> from zope.interface import directlyProvides
        >>> from zope.interface.interfaces import IComponentLookup
        >>> directlyProvides(sm, IComponentLookup)
        >>> folder.setSiteManager(sm)
        >>> directlyProvides(folder, zope.component.interfaces.ISite)
        >>> subs = list(FolderSublocations(folder).sublocations())
        >>> subs.remove(folder['ob1'])
        >>> subs.remove(folder['ob2'])
        >>> subs.remove(folder['ob3'])
        >>> subs.remove(sm)
        >>> subs
        []
    """

    def __init__(self, folder):
        self.folder = folder

    def sublocations(self):
        folder = self.folder
        for key in folder:
            yield folder[key]

        if zope.component.interfaces.ISite.providedBy(folder):
            yield folder.getSiteManager()



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Registration Tests
"""
__docformat__ = "reStructuredText"

import doctest
import unittest

import zope.interface
import zope.interface.verify
from zope.component.interfaces import IPossibleSite
from zope.component.interfaces import ISite

from zope import site
from zope.site import folder
from zope.site import interfaces
from zope.site import testing


@zope.interface.implementer(interfaces.ILocalSiteManager)
class SiteManagerStub(object):
    pass


class CustomFolder(folder.Folder):

    def __init__(self, name):
        self.__name__ = name
        super(CustomFolder, self).__init__()

    def __repr__(self):  # pragma: no cover
        return '<%s %s>' % (self.__class__.__name__, self.__name__)


def test_SiteManagerAdapter():
    """
    The site manager adapter is used to find the nearest site for any given
    location. If the provided context is a site,

      >>> site = folder.Folder()
      >>> sm = SiteManagerStub()
      >>> site.setSiteManager(sm)

    then the adapter simply return's the site's site manager:

      >>> from zope.site import SiteManagerAdapter
      >>> SiteManagerAdapter(site) is sm
      True

    If the context is a location (i.e. has a `__parent__` attribute),

      >>> ob = folder.Folder()
      >>> ob.__parent__ = site
      >>> ob2 = folder.Folder()
      >>> ob2.__parent__ = ob

    we 'acquire' the closest site and return its site manager:

      >>> SiteManagerAdapter(ob) is sm
      True
      >>> SiteManagerAdapter(ob2) is sm
      True

    If we are unable to find a local site manager, then the global site
    manager is returned.

      >>> import zope.component
      >>> orphan = CustomFolder('orphan')
      >>> SiteManagerAdapter(orphan) is zope.component.getGlobalSiteManager()
      True
    """


class TestSiteManagerContainer(unittest.TestCase):
    """This test is for objects that don't have site managers by
    default and that always give back the site manager they were
    given.

    Subclasses need to define a method, 'makeTestObject', that takes no
    arguments and that returns a new site manager
    container that has no site manager."""

    def makeTestObject(self):
        from zope.site import SiteManagerContainer
        return SiteManagerContainer()

    def test_IPossibleSite_verify(self):
        zope.interface.verify.verifyObject(IPossibleSite,
                                           self.makeTestObject())

    def test_get_and_set(self):
        smc = self.makeTestObject()
        self.assertFalse(ISite.providedBy(smc))
        sm = site.LocalSiteManager(smc)
        smc.setSiteManager(sm)
        self.assertTrue(ISite.providedBy(smc))
        self.assertTrue(smc.getSiteManager() is sm)
        zope.interface.verify.verifyObject(ISite, smc)

    def test_set_w_bogus_value(self):
        smc = self.makeTestObject()
        self.assertRaises(Exception, smc.setSiteManager, self)

    def test_get_no_manager(self):
        smc = self.makeTestObject()
        with self.assertRaises(LookupError):
            smc.getSiteManager()


class TestSMFolderFactory(unittest.TestCase):

    def test_call(self):
        from zope.site.site import SiteManagementFolder
        from zope.site.site import SMFolderFactory
        result = SMFolderFactory(None)('')
        self.assertIsInstance(result, SiteManagementFolder)


class TestSubscriber(unittest.TestCase):

    def getSiteManager(self):
        return self

    def test_set_and_clear(self):
        from zope.component.hooks import getSite

        from zope.site.site import clearThreadSiteSubscriber
        from zope.site.site import threadSiteSubscriber

        threadSiteSubscriber(self, None)
        self.assertIs(self, getSite())
        clearThreadSiteSubscriber(None)
        self.assertIsNone(getSite())

    def test_removed(self):
        from zope.site.site import ComponentLookupError
        from zope.site.site import siteManagerContainerRemoved
        siteManagerContainerRemoved(self, None)

        # And a raise is ignored too
        def gsm():
            raise ComponentLookupError()

        self.getSiteManager = gsm
        siteManagerContainerRemoved(self, None)


class TestTesting(unittest.TestCase):

    def setUp(self):
        self.sm = testing.siteSetUp(True)

    def tearDown(self):
        testing.siteTearDown()

    def test_add_utility(self):
        testing.addUtility(self.sm.getSiteManager(),
                           'Name',
                           zope.interface.Interface,
                           self)
        self.assertIn('Name', self.sm.getSiteManager()['default'])


def setUp(test):
    testing.siteSetUp()


def tearDown(test):
    testing.siteTearDown()


class Layer(object):

    @staticmethod
    def setUp():
        pass


def test_suite():
    site_suite = doctest.DocFileSuite('../site.rst',
                                      setUp=setUp, tearDown=tearDown)
    # XXX Isolate the site.txt tests within their own layer as they do some
    # component registration.
    site_suite.layer = Layer

    return unittest.TestSuite((
        doctest.DocTestSuite(),
        site_suite,
        unittest.defaultTestLoader.loadTestsFromName(__name__),
    ))



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.site package
"""
import os

from setuptools import find_packages
from setuptools import setup


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


TESTS_REQUIRE = [
    'ZODB',
    'zope.component[zcml]',
    'zope.configuration',
    'zope.security[zcml]',
    'zope.testing',
    'zope.testrunner',
]

setup(
    name='zope.site',
    version='4.6',
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.org',
    description='Local registries for zope component architecture',
    long_description=(
        read('README.rst')
        + '\n\n' +
        read('src', 'zope', 'site', 'site.rst')
        + '\n\n' +
        read('CHANGES.rst')
    ),
    keywords="zope component architecture local",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Zope :: 3',
    ],
    url='http://zopesite.readthedocs.io',
    license='ZPL 2.1',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['zope'],
    extras_require={
        'test': TESTS_REQUIRE,
        'docs': [
            'sphinx',
            'repoze.sphinx.autointerface',
            'sphinx_rtd_theme',
        ],
    },
    install_requires=[
        'setuptools',
        'zope.annotation',
        'zope.container',
        'zope.deprecation',
        'zope.security',
        'zope.component >= 4.5.0',
        'zope.event',
        'zope.interface >= 4.5.0',
        'zope.lifecycleevent',
        'zope.location',
    ],
    tests_require=TESTS_REQUIRE,
    include_package_data=True,
    zip_safe=False,
    python_requires=', '.join([
        '>=2.7',
        '!=3.0.*',
        '!=3.1.*',
        '!=3.2.*',
        '!=3.3.*',
        '!=3.4.*',
    ]),
)



#############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
"""

import unittest

import zope.component
import zope.container.testing
from zope.component import getSiteManager
from zope.event import notify
from zope.lifecycleevent import ObjectRemovedEvent
from zope.lifecycleevent.interfaces import IObjectRemovedEvent

import zope.site.testing
from zope.site.folder import rootFolder
from zope.site.site import SiteManagerContainer


class Dummy(object):
    pass


removed_called = False


def removed_event(obj, event):
    global removed_called
    removed_called = True


def dispatch_event(obj, event):
    sm = obj._sm
    if sm is not None:
        for k, v in sm.items():
            notify(ObjectRemovedEvent(v, sm, k))


class SiteManagerContainerTest(unittest.TestCase):

    def setUp(self):
        self.root = rootFolder()
        zope.site.testing.siteSetUp(self.root)

        global removed_called
        removed_called = False

        sm = getSiteManager()
        sm.registerHandler(removed_event, (Dummy, IObjectRemovedEvent))
        sm.registerHandler(
            dispatch_event, (SiteManagerContainer, IObjectRemovedEvent))

    def tearDown(self):
        zope.site.testing.siteTearDown()

    def test_delete_smc_should_propagate_removed_event(self):
        container = SiteManagerContainer()
        self.root['container'] = container

        zope.site.testing.createSiteManager(container)
        container.getSiteManager()['child'] = Dummy()

        del self.root['container']
        self.assertTrue(removed_called)

    def test_delete_when_smc_has_no_sitemanager(self):
        container = SiteManagerContainer()
        self.root['container'] = container

        del self.root['container']


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)
